<?php
/*
    This file serves up data for asynchronous requests (usually as JSON).
*/

require_once('config.php');
require_once('functions.php');

if (!isset($_GET['command'])) die();

$agency = $cfg['agency']['shortname'];

if ($_GET['command'] == "vehicleLocations") {
    // return a JSON array of vehicles and their locations.

    $db = dbc();

    // MAX age of a bus location to return
    $maxAge = $cfg['agency']['staleBusTime'];

    $time = 0;
    if (isset($_GET['t'])) $time = $db->real_escape_string((int)$_GET['t']);
    if ($time < $maxAge) $time = time() - $maxAge;
    $sql = <<< V_LOC
        SELECT
        `id`, `title` AS `route`, `routeTag`, `dirTag`, `lat`, `lon`, `heading`, `speedKmHr`
        FROM `vehicleLocations`
        LEFT JOIN `routes` ON `vehicleLocations`.`routeTag` = `routes`.`tag`
        WHERE `time` > '{$time}'
        AND `vehicleLocations`.`agency` = '{$agency}'
V_LOC;
    $result = dbq($db, $sql);

    $locations = array();
    while($row = $result->fetch_assoc()) $locations[$row['id']] = $row;

    // predictions
    $sql = <<< V_LOC
        SELECT
        `predictions`.`vehicle`, `stops`.`title`, `predictions`.`epochTime`
        FROM `predictions`
        LEFT JOIN `stops` ON `predictions`.`stopTag` = `stops`.`tag`
        WHERE `predictions`.`agency` = '{$agency}'
        ORDER BY `epochTime` ASC
V_LOC;
    $result = dbq($db, $sql);

    while($row = $result->fetch_assoc()) if (isset($locations[$row['vehicle']])) $locations[$row['vehicle']]['stops'][] = $row;

    foreach ($locations as $key => $location) {
        $locations[$key]['route'] = cleanRouteName($location['route']);
        $locations[$key]['stops'] = array_slice($location['stops'], 0, 8);
    }

    $lastFetched = 0;
    $result = dbq($db, "SELECT `time` FROM `lastChecked` WHERE `selector` = 'vehicleLocations' LIMIT 1", true);
    if ($result->num_rows == 1) {
        $arr = $result->fetch_array();
        $lastFetched = $arr[0];
    }

    // also return lastTime!
    header("Content-type: application/json");
    echo json_encode(array(
        "vehicles" => $locations,
        "lastTime" => $lastFetched,
    ));

} elseif ($_GET['command'] == "routeConfig") {
    // return a JSON array of routes and stops.

    $db = dbc();

    $sql = "SELECT `tag`, `title` FROM `routes` WHERE `agency` = '{$agency}' AND `tag` NOT IN ('".implode($cfg['agency']['hideRoutes'], "','")."')";
    $result = dbq($db, $sql);
    $routes = array();
    while($row = $result->fetch_assoc()) $routes[$row['tag']] = $row;

    $sql = "SELECT `tag`, `title`, `lat`, `lon`, `stopId` FROM `stops` WHERE `agency` = '{$agency}'";
    $result = dbq($db, $sql);
    $stops = array();
    while($row = $result->fetch_assoc()) {
        $stops[$row['tag']] = $row;
        $stops[$row['tag']]['routes'] = array();
    }

    $sql = "SELECT `route`, `stop` FROM `route_stops` WHERE `agency` = '{$agency}' AND `route` NOT IN ('".implode($cfg['agency']['hideRoutes'], "','")."')";
    $result = dbq($db, $sql);
    while($row = $result->fetch_assoc()) $stops[$row['stop']]['routes'][$row['route']] = array();

    $sql = "SELECT `routes`.`tag`, `routes`.`title`, `stopTag`, `epochTime`, `vehicle` FROM `predictions` LEFT JOIN `routes` ON `predictions`.`routeTag` = `routes`.`tag` WHERE `routes`.`agency` = '{$agency}' AND `tag` NOT IN ('".implode($cfg['agency']['hideRoutes'], "','")."')";
    $result = dbq($db, $sql);
    while($row = $result->fetch_assoc()) $stops[$row['stopTag']]['routes'][$row['tag']][] = array("epochTime" => $row['epochTime'], "vehicle" => $row['vehicle']);

    // post proc
    foreach ($routes as $key => $route) {
        $routes[$key]['title'] = cleanRouteName($route['title']);
    }
    $unset = array();
    foreach ($stops as $key => $stop) {
        if (!in_array($key, $unset)) {
            foreach ($stops as $key2 => $stop2) {
                if ($key !== $key2 && $stop['title'] == $stop2['title']) {
                    $stops[$key]['routes'] = array_merge($stop['routes'], $stop2['routes']);
                    $unset[] = $key2;
                }
            }
        }
    }
    foreach ($unset as $un) unset($stops[$un]);

    $lastFetched = 0;
    $result = dbq($db, "SELECT `time` FROM `lastChecked` WHERE `selector` = 'routeConfig' LIMIT 1", true);
    if ($result->num_rows == 1) {
        $arr = $result->fetch_array();
        $lastFetched = $arr[0];
    }

    // also return lastTime!
    header("Content-type: application/json");
    echo json_encode(array(
        "routes" => $routes,
        "stops" => $stops,
        "lastTime" => $lastFetched,
    ));
}


?>
